@@ -7,24 +7,38 @@ import textwrap
 import time
 
 import click
-import six
 import yaml
 
 from dagster import PipelineDefinition, check, execute_pipeline
-from dagster.cli.load_handle import handle_for_pipeline_cli_args, handle_for_repo_cli_args
-from dagster.cli.load_snapshot import get_pipeline_snapshot_from_cli_args
-from dagster.core.definitions import ExecutionTargetHandle
+from dagster.cli.workspace.cli_target import (
+    get_external_pipeline_from_kwargs,
+    get_external_repository_from_kwargs,
+    pipeline_target_argument,
+    repository_target_argument,
+)
+from dagster.core.definitions.executable import ExecutablePipeline
 from dagster.core.definitions.partition import PartitionScheduleDefinition
+from dagster.core.host_representation import InProcessRepositoryLocation
 from dagster.core.instance import DagsterInstance
 from dagster.core.snap import PipelineSnapshot, SolidInvocationSnap
 from dagster.core.storage.pipeline_run import PipelineRun
-from dagster.core.telemetry import telemetry_wrapper
+from dagster.core.telemetry import log_repo_stats, telemetry_wrapper
 from dagster.core.utils import make_new_backfill_id
 from dagster.seven import IS_WINDOWS, JSONDecodeError, json
-from dagster.utils import DEFAULT_REPOSITORY_YAML_FILENAME, load_yaml_from_glob_list, merge_dicts
+from dagster.utils import (
+    DEFAULT_REPOSITORY_YAML_FILENAME,
+    DEFAULT_WORKSPACE_YAML_FILENAME,
+    load_yaml_from_glob_list,
+    merge_dicts,
+)
+from dagster.utils.backcompat import canonicalize_backcompat_args
 from dagster.utils.error import serializable_error_info_from_exc_info
+from dagster.utils.hosted_user_process import (
+    recon_pipeline_from_origin,
+    recon_repo_from_external_repo,
+    repository_def_from_repository_handle,
+)
 from dagster.utils.indenting_printer import IndentingPrinter
-from dagster.visualize import build_graphviz_graph
 
 from .config_scaffolder import scaffold_pipeline_config
 
@@ -33,7 +47,6 @@ def create_pipeline_cli_group():
     group = click.Group(name="pipeline")
     group.add_command(pipeline_list_command)
     group.add_command(pipeline_print_command)
-    group.add_command(pipeline_graphviz_command)
     group.add_command(pipeline_execute_command)
     group.add_command(pipeline_backfill_command)
     group.add_command(pipeline_scaffold_command)
@@ -41,10 +54,8 @@ def create_pipeline_cli_group():
     return group
 
 
-REPO_TARGET_WARNING = (
-    'Can only use ONE of --repository-yaml/-y, --python-file/-f, --module-name/-m.'
-)
-REPO_ARG_NAMES = ['repository_yaml', 'module_name', 'fn_name', 'python_file']
+WORKSPACE_TARGET_WARNING = 'Can only use ONE of --workspace/-w, --python-file/-f, --module-name/-m.'
+WORKSPACE_ARG_NAMES = ['workspace', 'module_name', 'python_file', 'attribute']
 
 
 def apply_click_params(command, *click_params):
@@ -53,9 +64,12 @@ def apply_click_params(command, *click_params):
     return command
 
 
-def repository_target_argument(f):
+def legacy_repository_target_argument(f):
     return apply_click_params(
         f,
+        click.option(
+            '--workspace', '-w', type=click.Path(exists=True), help=('Path to workspace file')
+        ),
         click.option(
             '--repository-yaml',
             '-y',
@@ -78,32 +92,9 @@ def repository_target_argument(f):
     )
 
 
-def pipeline_target_command(f):
-    # f = repository_config_argument(f)
-    # nargs=-1 is used right now to make this argument optional
-    # it can only handle 0 or 1 pipeline names
-    # see .pipeline.create_pipeline_from_cli_args
-    return apply_click_params(
-        f,
-        click.option(
-            '--repository-yaml',
-            '-y',
-            type=click.Path(exists=True),
-            help=(
-                'Path to config file. Defaults to ./{default_filename} if --python-file '
-                'and --module-name are not specified'
-            ).format(default_filename=DEFAULT_REPOSITORY_YAML_FILENAME),
-        ),
-        click.argument('pipeline_name', nargs=-1),
-        click.option('--python-file', '-f', type=click.Path(exists=True)),
-        click.option('--module-name', '-m'),
-        click.option('--fn-name', '-n'),
-    )
-
-
 @click.command(
     name='list',
-    help="List the pipelines in a repository. {warning}".format(warning=REPO_TARGET_WARNING),
+    help="List the pipelines in a repository. {warning}".format(warning=WORKSPACE_TARGET_WARNING),
 )
 @repository_target_argument
 def pipeline_list_command(**kwargs):
@@ -111,8 +102,10 @@ def pipeline_list_command(**kwargs):
 
 
 def execute_list_command(cli_args, print_fn):
-    repository = handle_for_repo_cli_args(cli_args).build_repository_definition()
-
+    external_repository = get_external_repository_from_kwargs(cli_args)
+    # We should move this to use external repository
+    # https://github.com/dagster-io/dagster/issues/2556
+    repository = repository_def_from_repository_handle(external_repository.handle)
     title = 'Repository {name}'.format(name=repository.name)
     print_fn(title)
     print_fn('*' * len(title))
@@ -143,31 +136,27 @@ def format_description(desc, indent):
     return filled
 
 
-def create_pipeline_from_cli_args(kwargs):
-    return handle_for_pipeline_cli_args(kwargs).build_pipeline_definition()
-
-
 def get_pipeline_instructions(command_name):
     return (
         'This commands targets a pipeline. The pipeline can be specified in a number of ways:'
-        '\n\n1. dagster {command_name} <<pipeline_name>> (works if .{default_filename} exists)'
-        '\n\n2. dagster {command_name} <<pipeline_name>> -y path/to/{default_filename}'
-        '\n\n3. dagster {command_name} -f /path/to/file.py -n define_some_pipeline'
-        '\n\n4. dagster {command_name} -m a_module.submodule  -n define_some_pipeline'
-        '\n\n5. dagster {command_name} -f /path/to/file.py -n define_some_repo <<pipeline_name>>'
-        '\n\n6. dagster {command_name} -m a_module.submodule -n define_some_repo <<pipeline_name>>'
-    ).format(command_name=command_name, default_filename=DEFAULT_REPOSITORY_YAML_FILENAME)
+        '\n\n1. dagster pipeline {command_name} -p <<pipeline_name>> (works if .{default_filename} exists)'
+        '\n\n2. dagster pipeline {command_name} -p <<pipeline_name>> -w path/to/{default_filename}'
+        '\n\n3. dagster pipeline {command_name} -f /path/to/file.py -a define_some_pipeline'
+        '\n\n4. dagster pipeline {command_name} -m a_module.submodule -a define_some_pipeline'
+        '\n\n5. dagster pipeline {command_name} -f /path/to/file.py -a define_some_repo -p <<pipeline_name>>'
+        '\n\n6. dagster pipeline {command_name} -m a_module.submodule -a define_some_repo -p <<pipeline_name>>'
+    ).format(command_name=command_name, default_filename=DEFAULT_WORKSPACE_YAML_FILENAME)
 
 
 def get_partitioned_pipeline_instructions(command_name):
     return (
         'This commands targets a partitioned pipeline. The pipeline and partition set must be '
         'defined in a repository, which can be specified in a number of ways:'
-        '\n\n1. dagster {command_name} <<pipeline_name>> (works if .{default_filename} exists)'
-        '\n\n2. dagster {command_name} <<pipeline_name>> -y path/to/{default_filename}'
-        '\n\n3. dagster {command_name} -f /path/to/file.py -n define_some_repo <<pipeline_name>>'
-        '\n\n4. dagster {command_name} -m a_module.submodule -n define_some_repo <<pipeline_name>>'
-    ).format(command_name=command_name, default_filename=DEFAULT_REPOSITORY_YAML_FILENAME)
+        '\n\n1. dagster pipeline {command_name} -p <<pipeline_name>> (works if .{default_filename} exists)'
+        '\n\n2. dagster pipeline {command_name} -p <<pipeline_name>> -w path/to/{default_filename}'
+        '\n\n3. dagster pipeline {command_name} -f /path/to/file.py -a define_some_repo -p <<pipeline_name>>'
+        '\n\n4. dagster pipeline {command_name} -m a_module.submodule -a define_some_repo -p <<pipeline_name>>'
+    ).format(command_name=command_name, default_filename=DEFAULT_WORKSPACE_YAML_FILENAME)
 
 
 @click.command(
@@ -177,14 +166,14 @@ def get_partitioned_pipeline_instructions(command_name):
     ),
 )
 @click.option('--verbose', is_flag=True)
-@click.option('--image', type=click.STRING, help="Built image name:tag that holds user code.")
-@pipeline_target_command
+@pipeline_target_argument
 def pipeline_print_command(verbose, **cli_args):
     return execute_print_command(verbose, cli_args, click.echo)
 
 
 def execute_print_command(verbose, cli_args, print_fn):
-    pipeline_snapshot = get_pipeline_snapshot_from_cli_args(cli_args)
+    external_pipeline = get_external_pipeline_from_kwargs(cli_args)
+    pipeline_snapshot = external_pipeline.pipeline_snapshot
 
     if verbose:
         print_pipeline(pipeline_snapshot, print_fn=print_fn)
@@ -243,50 +232,33 @@ def print_solid(printer, pipeline_snapshot, solid_invocation_snap):
             printer.line(output_def_snap.name)
 
 
-@click.command(
-    name='graphviz',
-    help=(
-        'Visualize a pipeline using graphviz. Must be installed on your system '
-        '(e.g. homebrew install graphviz on mac). \n\n{instructions}'.format(
-            instructions=get_pipeline_instructions('graphviz')
-        )
-    ),
-)
-@click.option('--only-solids', is_flag=True)
-@pipeline_target_command
-def pipeline_graphviz_command(only_solids, **kwargs):
-    pipeline = create_pipeline_from_cli_args(kwargs)
-    build_graphviz_graph(pipeline, only_solids).view(cleanup=True)
-
-
 @click.command(
     name='execute',
     help='Execute a pipeline.\n\n{instructions}'.format(
         instructions=get_pipeline_instructions('execute')
     ),
 )
-@pipeline_target_command
+@pipeline_target_argument
 @click.option(
-    '-e',
-    '--env',
+    '-c',
+    '--config',
     type=click.Path(exists=True),
     multiple=True,
     help=(
-        'Specify one or more environment files. These can also be file patterns. '
-        'If more than one environment file is captured then those files are merged. '
-        'Files listed first take precendence. They will smash the values of subsequent '
+        'Specify one or more run config files. These can also be file patterns. '
+        'If more than one run config file is captured then those files are merged. '
+        'Files listed first take precedence. They will smash the values of subsequent '
         'files at the key-level granularity. If the file is a pattern then you must '
         'enclose it in double quotes'
         '\n\nExample: '
-        'dagster pipeline execute pandas_hello_world -e "pandas_hello_world/*.yaml"'
-        '\n\nYou can also specifiy multiple files:'
+        'dagster pipeline execute -p pandas_hello_world -c "pandas_hello_world/*.yaml"'
+        '\n\nYou can also specify multiple files:'
         '\n\nExample: '
-        'dagster pipeline execute pandas_hello_world -e pandas_hello_world/solids.yaml '
+        'dagster pipeline execute -p pandas_hello_world -c pandas_hello_world/solids.yaml '
         '-e pandas_hello_world/env.yaml'
     ),
 )
 @click.option(
-    '-p',
     '--preset',
     type=click.STRING,
     help='Specify a preset to use for this pipeline. Presets are defined on pipelines under '
@@ -296,8 +268,46 @@ def pipeline_graphviz_command(only_solids, **kwargs):
     '-d', '--mode', type=click.STRING, help='The name of the mode in which to execute the pipeline.'
 )
 @click.option('--tags', type=click.STRING, help='JSON string of tags to use for this pipeline run')
+@click.option(
+    '-s',
+    '--solid-selection',
+    type=click.STRING,
+    help=(
+        'Specify the solid subselection to execute. It can be multiple clauses separated by commas.'
+        'Examples:'
+        '\n- "some_solid" will execute "some_solid" itself'
+        '\n- "*some_solid" will execute "some_solid" and all its ancestors (upstream dependencies)'
+        '\n- "*some_solid+++" will execute "some_solid", all its ancestors, and its descendants'
+        '   (downstream dependencies) within 3 levels down'
+        '\n- "*some_solid,other_solid_a,other_solid_b+" will execute "some_solid" and all its'
+        '   ancestors, "other_solid_a" itself, and "other_solid_b" and its direct child solids'
+    ),
+)
+@click.option(
+    # backcompat
+    '-e',
+    '--env',
+    type=click.Path(exists=True),
+    multiple=True,
+    help=(
+        'Please use -c, --config to specify one or more run config files. '
+        '-e, --env is deprecated and will be removed in 0.9.0.'
+    ),
+)
 @telemetry_wrapper
-def pipeline_execute_command(env, preset, mode, **kwargs):
+def pipeline_execute_command(config, preset, mode, **kwargs):
+    env = (
+        canonicalize_backcompat_args(
+            (config if config else None),
+            '--config',
+            (kwargs.get('env') if kwargs.get('env') else None),
+            '--env',
+            '0.9.0',
+            stacklevel=2,  # this stacklevel can point the warning to this line
+        )
+        or tuple()  # back to default empty tuple
+    )
+
     check.invariant(isinstance(env, tuple))
 
     if preset:
@@ -308,18 +318,28 @@ def pipeline_execute_command(env, preset, mode, **kwargs):
     env = list(env)
     tags = get_tags_from_args(kwargs)
 
-    execute_execute_command(env, kwargs, mode, tags)
+    result = execute_execute_command(env, kwargs, mode, tags)
+    if not result.success:
+        raise click.ClickException('Pipeline run {} resulted in failure.'.format(result.run_id))
 
 
 def execute_execute_command(env, cli_args, mode=None, tags=None):
-    pipeline = create_pipeline_from_cli_args(cli_args)
-    return do_execute_command(pipeline, env, mode, tags)
+    external_pipeline = get_external_pipeline_from_kwargs(cli_args)
+    # We should move this to use external pipeline
+    # https://github.com/dagster-io/dagster/issues/2556
+    pipeline = recon_pipeline_from_origin(external_pipeline.handle.get_origin())
+    solid_selection = get_solid_selection_from_args(cli_args)
+    return do_execute_command(pipeline, env, mode, tags, solid_selection)
 
 
 def execute_execute_command_with_preset(preset_name, cli_args, _mode):
-    pipeline = handle_for_pipeline_cli_args(cli_args).build_pipeline_definition()
-    cli_args.pop('pipeline_name')
+    external_pipeline = get_external_pipeline_from_kwargs(cli_args)
+    # We should move this to use external pipeline
+    # https://github.com/dagster-io/dagster/issues/2556
+    pipeline = recon_pipeline_from_origin(external_pipeline.handle.get_origin())
+
     tags = get_tags_from_args(cli_args)
+    solid_selection = get_solid_selection_from_args(cli_args)
 
     return execute_pipeline(
         pipeline,
@@ -327,22 +347,24 @@ def execute_execute_command_with_preset(preset_name, cli_args, _mode):
         instance=DagsterInstance.get(),
         raise_on_error=False,
         tags=tags,
+        solid_selection=solid_selection,
     )
 
 
-def do_execute_command(pipeline_def, env_file_list, mode=None, tags=None):
-    check.inst_param(pipeline_def, 'pipeline_def', PipelineDefinition)
+def do_execute_command(pipeline, env_file_list, mode=None, tags=None, solid_selection=None):
+    check.inst_param(pipeline, 'pipeline', ExecutablePipeline)
     env_file_list = check.opt_list_param(env_file_list, 'env_file_list', of_type=str)
 
-    environment_dict = load_yaml_from_glob_list(env_file_list) if env_file_list else {}
+    run_config = load_yaml_from_glob_list(env_file_list) if env_file_list else {}
 
     return execute_pipeline(
-        pipeline_def,
-        environment_dict=environment_dict,
+        pipeline,
+        run_config=run_config,
         mode=mode,
         tags=tags,
         instance=DagsterInstance.get(),
         raise_on_error=False,
+        solid_selection=solid_selection,
     )
 
 
@@ -352,30 +374,28 @@ def do_execute_command(pipeline_def, env_file_list, mode=None, tags=None):
         instructions=get_pipeline_instructions('launch')
     ),
 )
-@pipeline_target_command
+@pipeline_target_argument
 @click.option(
-    '-e',
-    '--env',
+    '-c',
+    '--config',
     type=click.Path(exists=True),
     multiple=True,
     help=(
-        'Specify one or more environment files. These can also be file patterns. '
-        'If more than one environment file is captured then those files are merged. '
-        'Files listed first take precendence. They will smash the values of subsequent '
+        'Specify one or more run config files. These can also be file patterns. '
+        'If more than one run config file is captured then those files are merged. '
+        'Files listed first take precedence. They will smash the values of subsequent '
         'files at the key-level granularity. If the file is a pattern then you must '
         'enclose it in double quotes'
         '\n\nExample: '
-        'dagster pipeline execute pandas_hello_world -e "pandas_hello_world/*.yaml"'
-        '\n\nYou can also specifiy multiple files:'
+        'dagster pipeline launch pandas_hello_world -c "pandas_hello_world/*.yaml"'
+        '\n\nYou can also specify multiple files:'
         '\n\nExample: '
-        'dagster pipeline execute pandas_hello_world -e pandas_hello_world/solids.yaml '
+        'dagster pipeline launch pandas_hello_world -c pandas_hello_world/solids.yaml '
         '-e pandas_hello_world/env.yaml'
     ),
 )
 @click.option(
-    '-p',
     '--preset-name',
-    '--preset',
     type=click.STRING,
     help='Specify a preset to use for this pipeline. Presets are defined on pipelines under '
     'preset_defs.',
@@ -384,37 +404,107 @@ def do_execute_command(pipeline_def, env_file_list, mode=None, tags=None):
     '-d', '--mode', type=click.STRING, help='The name of the mode in which to execute the pipeline.'
 )
 @click.option('--tags', type=click.STRING, help='JSON string of tags to use for this pipeline run')
+@click.option(
+    '-s',
+    '--solid-selection',
+    type=click.STRING,
+    help=(
+        'Specify the solid subselection to launch. It can be multiple clauses separated by commas.'
+        'Examples:'
+        '\n- "some_solid" will launch "some_solid" itself'
+        '\n- "*some_solid" will launch "some_solid" and all its ancestors (upstream dependencies)'
+        '\n- "*some_solid+++" will launch "some_solid", all its ancestors, and its descendants'
+        '   (downstream dependencies) within 3 levels down'
+        '\n- "*some_solid,other_solid_a,other_solid_b+" will launch "some_solid" and all its'
+        '   ancestors, "other_solid_a" itself, and "other_solid_b" and its direct child solids'
+    ),
+)
+@click.option(
+    # backcompat
+    '-e',
+    '--env',
+    type=click.Path(exists=True),
+    multiple=True,
+    help=(
+        'Please use -c, --config to specify one or more run config files. '
+        '-e, --env is deprecated and will be removed in 0.9.0.'
+    ),
+)
 @telemetry_wrapper
-def pipeline_launch_command(env, preset_name, mode, **kwargs):
+def pipeline_launch_command(config, preset_name, mode, **kwargs):
+    env = (
+        canonicalize_backcompat_args(
+            (config if config else None),
+            '--config',
+            (kwargs.get('env') if kwargs.get('env') else None),
+            '--env',
+            '0.9.0',
+            stacklevel=2,  # this stacklevel can point the warning to this line
+        )
+        or tuple()  # back to default empty tuple
+    )
+
     env = list(check.opt_tuple_param(env, 'env', default=(), of_type=str))
 
-    pipeline = create_pipeline_from_cli_args(kwargs)
+    external_pipeline = get_external_pipeline_from_kwargs(kwargs)
+    # We should move this to use external pipeline
+    # https://github.com/dagster-io/dagster/issues/2556
+    pipeline = recon_pipeline_from_origin(external_pipeline.get_origin())
 
     instance = DagsterInstance.get()
+    log_repo_stats(instance=instance, pipeline=pipeline, source='pipeline_launch_command')
 
     if preset_name:
         if env:
-            raise click.UsageError('Can not use --preset with --env.')
+            raise click.UsageError('Can not use --preset with --config.')
 
         if mode:
             raise click.UsageError('Can not use --preset with --mode.')
 
-        preset = pipeline.get_preset(preset_name)
+        preset = pipeline.get_definition().get_preset(preset_name)
     else:
         preset = None
 
     run_tags = get_tags_from_args(kwargs)
 
-    # FIXME need to check the env against environment_dict
+    solid_selection = get_solid_selection_from_args(kwargs)
+
+    if preset and preset.solid_selection is not None:
+        check.invariant(
+            solid_selection is None or solid_selection == preset.solid_selection,
+            'The solid_selection set in preset \'{preset}\', {preset_subset}, does not agree with '
+            'the `solid_selection` argument: {solid_selection}'.format(
+                preset=preset,
+                preset_subset=preset.solid_selection,
+                solid_selection=solid_selection,
+            ),
+        )
+        solid_selection = preset.solid_selection
+
+    # generate pipeline subset from the given solid_selection
+    if solid_selection:
+        pipeline = pipeline.subset_for_execution(solid_selection)
+
+    # FIXME need to check the env against run_config
     pipeline_run = instance.create_run_for_pipeline(
-        pipeline_def=pipeline,
-        solid_subset=preset.solid_subset if preset else None,
-        environment_dict=preset.environment_dict if preset else load_yaml_from_glob_list(env),
+        pipeline_def=pipeline.get_definition(),
+        solid_selection=solid_selection,
+        solids_to_execute=pipeline.solids_to_execute,
+        run_config=preset.run_config if preset else load_yaml_from_glob_list(env),
         mode=(preset.mode if preset else mode) or 'default',
         tags=run_tags,
     )
 
-    return instance.launch_run(pipeline_run.run_id)
+    recon_repo = pipeline.get_reconstructable_repository()
+
+    repo_location = InProcessRepositoryLocation(recon_repo)
+    external_pipeline = (
+        repo_location.get_repository(recon_repo.get_definition().name).get_full_external_pipeline(
+            pipeline.get_definition().name
+        ),
+    )
+
+    return instance.launch_run(pipeline_run.run_id, external_pipeline)
 
 
 @click.command(
@@ -423,16 +513,19 @@ def pipeline_launch_command(env, preset_name, mode, **kwargs):
         instructions=get_pipeline_instructions('scaffold_config')
     ),
 )
-@pipeline_target_command
-@click.option('-p', '--print-only-required', default=False, is_flag=True)
+@pipeline_target_argument
+@click.option('--print-only-required', default=False, is_flag=True)
 def pipeline_scaffold_command(**kwargs):
     execute_scaffold_command(kwargs, click.echo)
 
 
 def execute_scaffold_command(cli_args, print_fn):
-    pipeline_def = create_pipeline_from_cli_args(cli_args)
+    external_pipeline = get_external_pipeline_from_kwargs(cli_args)
+    # We should move this to use external pipeline
+    # https://github.com/dagster-io/dagster/issues/2556
+    pipeline = recon_pipeline_from_origin(external_pipeline.get_origin())
     skip_non_required = cli_args['print_only_required']
-    do_scaffold_command(pipeline_def, print_fn, skip_non_required)
+    do_scaffold_command(pipeline.get_definition(), print_fn, skip_non_required)
 
 
 def do_scaffold_command(pipeline_def, printer, skip_non_required):
@@ -478,12 +571,6 @@ def gen_partitions_from_args(partition_set, kwargs):
     return partitions[start:end]
 
 
-def get_backfill_priority_from_args(kwargs):
-    if kwargs.get('celery_base_priority') is None:
-        return None
-    return int(kwargs.get('celery_base_priority'))
-
-
 def get_tags_from_args(kwargs):
     if kwargs.get('tags') is None:
         return {}
@@ -498,6 +585,14 @@ def get_tags_from_args(kwargs):
         )
 
 
+def get_solid_selection_from_args(kwargs):
+    solid_selection_str = kwargs.get('solid_selection')
+    if not check.is_str(solid_selection_str):
+        return None
+
+    return [ele.strip() for ele in solid_selection_str.split(',')] if solid_selection_str else None
+
+
 def print_partition_format(partitions, indent_level):
     if not IS_WINDOWS and sys.stdout.isatty():
         _, tty_width = os.popen('stty size', 'r').read().split()
@@ -532,26 +627,14 @@ def validate_partition_slice(partitions, name, value):
     return index if is_start else index + 1
 
 
-def get_partition_sets_for_handle(handle):
-    check.inst_param(handle, 'handle', ExecutionTargetHandle)
-    repo = handle.build_repository_definition()
-
-    return repo.partition_set_defs + [
-        schedule_def.get_partition_set()
-        for schedule_def in repo.schedule_defs
-        if isinstance(schedule_def, PartitionScheduleDefinition)
-    ]
-
-
 @click.command(
     name='backfill',
     help='Backfill a partitioned pipeline.\n\n{instructions}'.format(
         instructions=get_partitioned_pipeline_instructions('backfill')
     ),
 )
-@pipeline_target_command
+@pipeline_target_argument
 @click.option(
-    '-p',
     '--partitions',
     type=click.STRING,
     help='Comma-separated list of partition names that we want to backfill',
@@ -562,7 +645,7 @@ def get_partition_sets_for_handle(handle):
     help='The name of the partition set over which we want to backfill.',
 )
 @click.option(
-    '-a', '--all', type=click.STRING, help='Specify to select all partitions to backfill.',
+    '--all', type=click.STRING, help='Specify to select all partitions to backfill.',
 )
 @click.option(
     '--from',
@@ -582,17 +665,6 @@ def get_partition_sets_for_handle(handle):
         'dagster pipeline backfill log_daily_stats --to 20191201'
     ),
 )
-@click.option(
-    '--celery-base-priority',
-    type=click.STRING,
-    help=(
-        'Specify a base run priority for all runs kicked off by this backfill job. Only meaningful '
-        'if you are launching runs against a Celery executor on a messaging queue that supports '
-        'priority.'
-        '\n\nExample: '
-        'dagster pipeline backfill log_daily_stats --celery-base-priority -3'
-    ),
-)
 @click.option('--tags', type=click.STRING, help='JSON string of tags to use for this pipeline run')
 @click.option('--noprompt', is_flag=True)
 def pipeline_backfill_command(**kwargs):
@@ -600,45 +672,33 @@ def pipeline_backfill_command(**kwargs):
 
 
 def execute_backfill_command(cli_args, print_fn, instance=None):
-    pipeline_name = cli_args.pop('pipeline_name')
-    repo_args = {k: v for k, v in cli_args.items() if k in REPO_ARG_NAMES}
-    if pipeline_name and not isinstance(pipeline_name, six.string_types):
-        if len(pipeline_name) == 1:
-            pipeline_name = pipeline_name[0]
-
     instance = instance or DagsterInstance.get()
-    handle = handle_for_repo_cli_args(repo_args)
-    repository = handle.build_repository_definition()
+    external_pipeline = get_external_pipeline_from_kwargs(cli_args)
+    external_repository = get_external_repository_from_kwargs(cli_args)
+
+    # We should move this to use external repository
+    # https://github.com/dagster-io/dagster/issues/2556
+    recon_repo = recon_repo_from_external_repo(external_repository)
+    repo_def = recon_repo.get_definition()
+
     noprompt = cli_args.get('noprompt')
 
-    # check run launcher
-    if not instance.run_launcher:
-        raise click.UsageError(
-            'A run launcher must be configured before running a backfill. You can configure a run '
-            'launcher (e.g. dagster_graphql.launcher.RemoteDagitRunLauncher) in your instance '
-            '`dagster.yaml` settings. See '
-            'https://docs.dagster.io/docs/deploying/instance/ for more'
-            'information.'
-        )
-
-    # Resolve pipeline
-    if not pipeline_name and noprompt:
-        raise click.UsageError('No pipeline specified')
-    if not pipeline_name:
-        pipeline_name = click.prompt(
-            'Select a pipeline to backfill: {}'.format(', '.join(repository.pipeline_names))
-        )
-    repository = handle.build_repository_definition()
-    if not repository.has_pipeline(pipeline_name):
-        raise click.UsageError('No pipeline found named `{}`'.format(pipeline_name))
-
-    pipeline = repository.get_pipeline(pipeline_name)
+    pipeline_def = repo_def.get_pipeline(external_pipeline.name)
 
     # Resolve partition set
-    all_partition_sets = get_partition_sets_for_handle(handle)
-    pipeline_partition_sets = [x for x in all_partition_sets if x.pipeline_name == pipeline.name]
+    all_partition_sets = repo_def.partition_set_defs + [
+        schedule_def.get_partition_set()
+        for schedule_def in repo_def.schedule_defs
+        if isinstance(schedule_def, PartitionScheduleDefinition)
+    ]
+
+    pipeline_partition_sets = [
+        x for x in all_partition_sets if x.pipeline_name == pipeline_def.name
+    ]
     if not pipeline_partition_sets:
-        raise click.UsageError('No partition sets found for pipeline `{}`'.format(pipeline.name))
+        raise click.UsageError(
+            'No partition sets found for pipeline `{}`'.format(pipeline_def.name)
+        )
     partition_set_name = cli_args.get('partition_set')
     if not partition_set_name:
         if len(pipeline_partition_sets) == 1:
@@ -658,11 +718,8 @@ def execute_backfill_command(cli_args, print_fn, instance=None):
     # Resolve partitions to backfill
     partitions = gen_partitions_from_args(partition_set, cli_args)
 
-    # Resolve priority
-    celery_priority = get_backfill_priority_from_args(cli_args)
-
     # Print backfill info
-    print_fn('\n     Pipeline: {}'.format(pipeline.name))
+    print_fn('\n     Pipeline: {}'.format(pipeline_def.name))
     print_fn('Partition set: {}'.format(partition_set.name))
     print_fn('   Partitions: {}\n'.format(print_partition_format(partitions, indent_level=15)))
 
@@ -678,19 +735,18 @@ def execute_backfill_command(cli_args, print_fn, instance=None):
             PipelineRun.tags_for_backfill_id(backfill_id), get_tags_from_args(cli_args),
         )
 
-        # for backwards compatibility - remove once prezi switched over to using tags argument
-        if celery_priority is not None:
-            run_tags['dagster-celery/run_priority'] = celery_priority
-
         for partition in partitions:
             run = instance.create_run_for_pipeline(
-                pipeline_def=pipeline,
+                pipeline_def=pipeline_def,
                 mode=partition_set.mode,
-                solid_subset=partition_set.solid_subset,
-                environment_dict=partition_set.environment_dict_for_partition(partition),
+                solids_to_execute=frozenset(partition_set.solid_selection)
+                if partition_set and partition_set.solid_selection
+                else None,
+                run_config=partition_set.run_config_for_partition(partition),
                 tags=merge_dicts(partition_set.tags_for_partition(partition), run_tags),
             )
-            instance.launch_run(run.run_id)
+
+            instance.launch_run(run.run_id, external_pipeline)
             # Remove once we can handle synchronous execution... currently limited by sqlite
             time.sleep(0.1)
 
